using System;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Grid.Tutorial.BusinessData;
using Dapfor.Net.Grid.Tutorial.BusinessData.OrderBook;
using Dapfor.Net.Grid.Tutorial.Properties;
using Dapfor.Net.Theming;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Grid.Tutorial.Presentation
{
    public partial class OrderBookControl : UserControl
    {
        private int _dealId;
        readonly Random _random = new Random();

        public OrderBookControl()
        {
            InitializeComponent();


            grid.Filter = new Filter(OnFilterRow);

            //Bind grid to multiple data sources
            grid.DataSource = new MultipleDataSources(Provider.Instance.SingleOrders, Provider.Instance.Strategies);
            grid.Headers[0].FixedColumns.Count = 1;
            grid.Theme = Theme.Default;
        }

        #region Filtering

        private bool IsDealFiltered(Deal deal)
        {
            return !string.IsNullOrEmpty(tbProduct.Text.Trim()) 
                       ? !deal.Instrument.Isin.ToLower().Contains(tbProduct.Text.Trim().ToLower()) 
                       : false;
        }

        private bool IsOrderFiltered(Order order)
        {
            bool isFiltered = !cbSkipOrders.Checked 
                    ? (!string.IsNullOrEmpty(tbProduct.Text.Trim()) ? !order.Instrument.Isin.ToLower().Contains(tbProduct.Text.Trim().ToLower()) : false)
                    : true;

            if (isFiltered)
            {
                foreach (Deal deal in order.Deals)
                {
                    isFiltered = IsDealFiltered(deal);
                    if (!isFiltered)
                    {
                        break;
                    }
                }    
            }
            return isFiltered;
        }

        private bool OnFilterRow(Row row)
        {
            bool isFiltered = false;
            if (!string.IsNullOrEmpty(tbProduct.Text))
            {
                //Check for the deal
                Deal deal = row.DataObject as Deal;
                if (deal != null)
                {
                    isFiltered = IsDealFiltered(deal);
                }

                //Check for the order
                Order order = row.DataObject as Order;
                if (order != null)
                {
                    isFiltered = IsOrderFiltered(order);
                }

                //Check for the strategy
                Strategy strategy = row.DataObject as Strategy;
                if (strategy != null)
                {
                    foreach (Order item in strategy.Orders)
                    {
                        isFiltered = IsOrderFiltered(item);
                        if(!isFiltered)
                        {
                            break;
                        }
                    }    
                }
            }
            return isFiltered;
        }

        private void tbProduct_TextChanged(object sender, EventArgs e)
        {
            grid.FilterRefresh();
        }

        #endregion

        #region Conditional databinding

        private void RefreshDataSource(object sender, EventArgs e)
        {
            Object dataSource = grid.DataSource;
            grid.DataSource = null;
            grid.DataSource = dataSource;
        }

        private void grid_RowAdding(object sender, GridRowAddingEventArgs e)
        {
            Strategy basket = e.DataObject as Strategy;
            if (basket != null)
            {
                e.DataObject = basket.Orders;
                e.InsertionType = !cbSkipStrategies.Checked
                                      ? InsertionType.AsChildAndCollection
                                      : InsertionType.AsCollection;
            }
            Order order = e.DataObject as Order;
            if (order != null)
            {
                e.DataObject = order.Deals;
                e.InsertionType = !cbSkipOrders.Checked
                                      ? InsertionType.AsChildAndCollection
                                      : InsertionType.AsCollection;
            }
        }

        private void grid_RowAdded(object sender, GridRowEventArgs e)
        {
            e.Row.Expanded = true;
        }

        #endregion

        private void OnAddSingleOrder(object sender, EventArgs e)
        {
            Order order = new Order("Order #" + (Provider.Instance.SingleOrders.Count + 1),
                                    _random.Next()%2 == 0 ? Way.Buy : Way.Sell,
                                    _random.Next(50, 150) + _random.NextDouble(),
                                    _random.Next(100, 50000),
                                    Provider.Instance.Instruments[_random.Next(40)],
                                    null);
            
            Provider.Instance.SingleOrders.Add(order);
        }

        private void OnAddStrategy(object sender, EventArgs e)
        {
            Strategy strategy = new Strategy("Strategy #" + (Provider.Instance.Strategies.Count + 1));
            Provider.Instance.Strategies.Add(strategy);            

            for(int i = 0; i < _random.Next(1, 5); ++i)
            {
                Order order = new Order("Order #" + (10 * Provider.Instance.Strategies.Count + strategy.Orders.Count + 1),
                                        _random.Next() % 2 == 0 ? Way.Buy : Way.Sell,
                                        _random.Next(50, 150) + _random.NextDouble(),
                                        _random.Next(100, 50000),
                                        Provider.Instance.Instruments[_random.Next(40)],
                                        null);

                strategy.Orders.Add(order);
            }
        }

        private void OnAddDeal(object sender, EventArgs e)
        {
            //Add a deal to a single order
            if(Provider.Instance.SingleOrders.Count > 0)
            {
                Order order = Provider.Instance.SingleOrders[_random.Next(Provider.Instance.SingleOrders.Count)];
                Deal deal = new Deal(order,
                     "Deal #" + (++_dealId),
                     _random.Next(50, 150) + _random.NextDouble(),
                     _random.Next(50, 1000),
                     "Counterpart " + _random.Next(1, 100));
                order.Deals.Add(deal);
            }

            //Add a deal to an order of a strategy
            if (Provider.Instance.Strategies.Count > 0)
            {
                Strategy strategy = Provider.Instance.Strategies[_random.Next(Provider.Instance.Strategies.Count)];
                Order order = strategy != null && strategy.Orders.Count > 0 ? strategy.Orders[_random.Next(strategy.Orders.Count)] : null;
                if (order != null)
                {
                    Deal deal = new Deal(order,
                         "Deal #" + (++_dealId),
                         _random.Next(50, 150) + _random.NextDouble(),
                         _random.Next(50, 1000),
                         "Counterpart " + _random.Next(1, 100));
                    order.Deals.Add(deal);
                }
            }

            
        }

        private void grid_PaintCell(object sender, PaintCellEventArgs e)
        {
            //Icon
            if (e.Cell.Column != null && e.Cell.Column.Id == "Id")
            {
                e.ImageSettings.Alignment = ContentAlignment.MiddleLeft;

                if (e.Cell.Row.DataObject is Strategy)
                {
                    e.Image = Resources.folder_cubes;
                }
                if (e.Cell.Row.DataObject is Order)
                {
                    e.Image = Resources.document_pulse;
                }
                if (e.Cell.Row.DataObject is Deal)
                {
                    e.Image = Resources.bullet_ball_glass_green;
                }
            }

            //Background
            if (e.Cell.Row.DataObject is Strategy)
            {
                e.Appearance.BackColor = Color.FromArgb(253, 241, 219);
                e.Appearance.GradientEndBackColor = Color.FromArgb(241, 231, 200);
                e.Appearance.GradientEnabled = true;
                e.Appearance.GradientDirection = GradientDirection.Vertical;
            }
            if (e.Cell.Row.DataObject is Order)
            {
                e.Appearance.BackColor = Color.AliceBlue;
                e.Appearance.GradientEndBackColor = Color.FromArgb(214, 234, 254);
                e.Appearance.GradientEnabled = true;
                e.Appearance.GradientDirection = GradientDirection.Vertical;
            }
            if (e.Cell.Row.DataObject is Deal)
            {
                e.Appearance.BackColor = Color.FromArgb(241, 253, 219);
                e.Appearance.GradientEndBackColor = Color.FromArgb(231, 241, 200);
                e.Appearance.GradientEnabled = true;
                e.Appearance.GradientDirection = GradientDirection.Vertical;
            }
        }

        private void grid_RowExpansionChanging(object sender, GridRowExpansionEventArgs e)
        {
            Cell cell = e.Grid.HitTests.CellTest(e.Grid.PointToClient(Cursor.Position));
            if (cell != null && cell.Column != null && cell.Column.Id == "Status")
            {
                e.Cancel = true;
            }
        }
    }
}